// Adapted for LHAPDFv4 by Mike Whalley.
// Adapted for LHAPDFv5 by Craig Group & Mike Whalley.
// Fortran portability and string passing by Andy Buckley.

#include "LHAPDF/LHAPDFWrap.h"
#include <iostream>
using namespace std;

void DEPRECATED() {
  cerr << "WARNING: Class-based C++ LHAPDF wrapper is deprecated. " 
       << "Please use the namespaced functions from LHAPDF.h instead." << endl;
}

LHAPDFWrap::LHAPDFWrap() {
  DEPRECATED();
}

LHAPDFWrap::LHAPDFWrap(const string& name, int memset) {
  DEPRECATED();
  initPDFSetByName(name);
  initPDF(memset);
}

LHAPDFWrap::LHAPDFWrap(const string& name) {
  DEPRECATED();
  initPDFSetByName(name);
}


//For multiple set use
LHAPDFWrap::LHAPDFWrap(int nset, const string& name, int memset) {
  DEPRECATED();
  initPDFSetByNameM(nset, name);
  initPDFM(nset, memset);
}

LHAPDFWrap::LHAPDFWrap(int nset, const string& name) {
  DEPRECATED();
  initPDFSetByNameM(nset, name);
}



void LHAPDFWrap::initPDFSet(const string& name) { 
  char cname[1000];
  strcpy(cname, name.c_str());
  finitpdfset_(cname, name.length());
}

void LHAPDFWrap::initPDFSetByName(const string& name) { 
  char cname[1000];
  strcpy(cname, name.c_str());
  finitpdfsetbyname_(cname, name.length());
}

void LHAPDFWrap::initPDF(int memset) { 
  finitpdf_(&memset); 
}

void LHAPDFWrap::getDescription() {
  fgetdesc_();
}



void LHAPDFWrap::initPDFSetM(int nset, const string& name) {
  char cname[1000];
  strcpy(cname, name.c_str());
  finitpdfsetm_(&nset, cname, name.length());
}

void LHAPDFWrap::initPDFSetByNameM(int nset, const string& name) {
  char cname[1000];
  strcpy(cname, name.c_str());
  finitpdfsetbynamem_(&nset, cname, name.length());
}

void LHAPDFWrap::initPDFM(int nset, int memset) {
  finitpdfm_(&nset, &memset);
}

void LHAPDFWrap::getDescriptionM(int nset) {
  fgetdescm_(&nset);
}


vector<double> LHAPDFWrap::xfx(double x, double Q) {  
  vector<double> r(13);
  fevolvepdf_(&x, &Q, &r[0]);
  return r;
}

double LHAPDFWrap::xfx(double x, double Q, int fl) {  
  vector<double> r(13);
  fevolvepdf_(&x, &Q, &r[0]);
  return r[fl+6];
}

vector<double> LHAPDFWrap::xfxp(double x, double Q, double P2, int ip) {  
  vector<double> r(13);
  fevolvepdfp_(&x, &Q, &P2, &ip, &r[0]);
  return r;
}

double LHAPDFWrap::xfxp(double x, double Q, double P2, int ip, int fl) {  
  vector<double> r(13);
  fevolvepdfp_(&x, &Q, &P2, &ip, &r[0]);
  return r[fl+6];
}

vector<double> LHAPDFWrap::xfxa(double x, double Q, double a) {  
  vector<double> r(13);
  fevolvepdfa_(&x, &Q, &a, &r[0]);
  return r;
}

double LHAPDFWrap::xfxa(double x, double Q, double a, int fl) {  
  vector<double> r(13);
  fevolvepdfa_(&x, &Q, &a, &r[0]);
  return r[fl+6];
}

vector<double> LHAPDFWrap::xfxphoton(double x, double Q) {  
  vector<double> r(13);  
  double mphoton;
  fevolvepdfphoton_(&x, &Q, &r[0], &mphoton );
  r.push_back(mphoton);
  return r;
}

double LHAPDFWrap::xfxphoton(double x, double Q, int fl) {  
  vector<double> r(13);
  double mphoton;
  fevolvepdfphoton_(&x, &Q, &r[0], &mphoton);
  if ( fl == 7 )
    return mphoton;
  else
    return r[fl+6];
}

int LHAPDFWrap::numberPDF() {
  int N;
  fnumberpdf_(&N); 
  return N;
}

double LHAPDFWrap::alphasPDF(double Q) {
  double a;
  falphaspdf_(&Q, &a);
  return a;
}

int LHAPDFWrap::getOrderPDF() {
  int N;
  fgetorderpdf_(&N);
  return N;
}

int LHAPDFWrap::getOrderAlphaS() {
  int N;
  fgetorderas_(&N);
  return N;
}

double LHAPDFWrap::getQMass(int f) {
  double m;
  fgetqmass_(&f, &m);
  return m;
}

double LHAPDFWrap::getThreshold(int f) {
  double m;
  fgetthreshold_(&f, &m);
  return m;
}

int LHAPDFWrap::getNf() {
  int N; 
  fgetnf_(&N); 
  return N;
}

double LHAPDFWrap::getLam4(int m){
  double l;
  fgetlam4_(&m, &l);
  return l;
}

double LHAPDFWrap::getLam5(int m){
  double l;
  fgetlam5_(&m, &l);
  return l;
}

double LHAPDFWrap::getXmin(int m){
  double xmin;
  fgetxmin_(&m, &xmin);
  return xmin;
}

double LHAPDFWrap::getXmax(int m){
  double xmax;
  fgetxmax_(&m, &xmax);
  return xmax;
}

double LHAPDFWrap::getQ2min(int m){
  double q2min;
  fgetq2min_(&m, &q2min);
  return q2min;
}

double LHAPDFWrap::getQ2max(int m){
  double q2max;
  fgetq2max_(&m, &q2max);
  return q2max;
}


// Functions below are the same as above but with M appended to their name.
// They are for multiple set use in LHAPDF.  They also take an extra paramater
// which is a label for their memory location 


vector<double> LHAPDFWrap::xfxM(int nset, double x, double Q) {  
  vector<double> r(13);
  fevolvepdfm_(&nset, &x, &Q, &r[0]);
  return r;
}

double LHAPDFWrap::xfxM(int nset, double x, double Q, int fl) {  
  vector<double> r(13);
  fevolvepdfm_(&nset, &x, &Q, &r[0]);
  return r[fl+6];
}

vector<double> LHAPDFWrap::xfxpM(int nset, double x, double Q, double P2, int ip) {  
  vector<double> r(13);
  fevolvepdfpm_(&nset, &x, &Q, &P2,&ip, &r[0]);
  return r;
}

double LHAPDFWrap::xfxpM(int nset, double x, double Q, double P2, int ip, int fl) {  
  vector<double> r(13);
  fevolvepdfpm_(&nset, &x, &Q, &P2, &ip, &r[0]);
  return r[fl+6];
}

vector<double> LHAPDFWrap::xfxaM(int nset, double x, double Q, double a) {  
  vector<double> r(13);
  fevolvepdfam_(&nset, &x, &Q, &a, &r[0]);
  return r;
}

double LHAPDFWrap::xfxaM(int nset, double x, double Q, double a, int fl) {  
  vector<double> r(13);
  fevolvepdfam_(&nset, &x, &Q, &a, &r[0]);
  return r[fl+6];
}

vector<double> LHAPDFWrap::xfxphotonM(int nset, double x, double Q) {  
  vector<double> r(13);
  double mphoton;
  fevolvepdfphotonm_(&nset, &x, &Q, &r[0], &mphoton );
  r.push_back(mphoton);
  return r;
}

double LHAPDFWrap::xfxphotonM(int nset, double x, double Q, int fl) {  
  vector<double> r(13);
  double mphoton;
  fevolvepdfphotonm_(&nset, &x, &Q, &r[0], &mphoton);
  if ( fl == 7 )
    return mphoton;
  else
    return r[fl+6];
}

int LHAPDFWrap::numberPDFM(int nset) {
  int N; 
  fnumberpdfm_(&nset, &N); 
  return N;
}

double LHAPDFWrap::alphasPDFM(int nset, double Q) {
  double a;
  falphaspdfm_(&nset, &Q, &a);
  return a;
}

int LHAPDFWrap::getOrderPDFM(int nset) {
  int N;
  fgetorderpdfm_(&nset, &N);
  return N;
}

int LHAPDFWrap::getOrderAlphaSM(int nset) {
  int N;
  fgetorderasm_(&nset, &N);
  return N;
}

double LHAPDFWrap::getQMassM(int nset, int f) {
  double m;
  fgetqmassm_(&nset, &f, &m);
  return m;
}

double LHAPDFWrap::getThresholdM(int nset, int f) {
  double m;
  fgetthresholdm_(&nset, &f, &m);
  return m;
}

int LHAPDFWrap::getNfM(int nset) {
  int N; 
  fgetnfm_(&nset, &N); 
  return N;
}

double LHAPDFWrap::getLam4M(int nset, int m){
  double l;
  fgetlam4m_(&nset, &m, &l);
  return l;
}

double LHAPDFWrap::getLam5M(int nset, int m){
  double l;
  fgetlam5m_(&nset, &m, &l);
  return l;
}

double LHAPDFWrap::getXminM(int nset,int m){
  double xmin;
  fgetxminm_(&nset, &m, &xmin);
  return xmin;
}

double LHAPDFWrap::getXmaxM(int nset,int m){
  double xmax;
  fgetxmaxm_(&nset, &m, &xmax);
  return xmax;
}

double LHAPDFWrap::getQ2minM(int nset,int m){
  double q2min;
  fgetq2minm_(&nset, &m, &q2min);
  return q2min;
}

double LHAPDFWrap::getQ2maxM(int nset,int m){
  double q2max;
  fgetq2maxm_(&nset, &m, &q2max);
  return q2max;
}
